Exception: BugBunny::RoutingError

Inherits:
NotFound show all
Defined in:
lib/bug_bunny/exception.rb

Overview

Error 404 específico de enrutamiento. Se lanza cuando el servicio remoto no tiene una ruta registrada para el verbo y path solicitados. Análogo a ‘ActionController::RoutingError` en Rails.

Examples:

rescue BugBunny::RoutingError => e
  e.message # => 'No route matches [GET] "secrets"'